package yaes.framework.entity.control;

import java.util.ArrayList;

import yaes.framework.world.map.AbstractMovingObject;
import yaes.framework.world.map.Location;
import yaes.framework.world.map.Speed;

public class AbstractSpeedControllable extends AbstractMovingObject implements
		ISpeedControllable {
	private final ArrayList<ISpeedControl> speedControls = new ArrayList<ISpeedControl>();

	public AbstractSpeedControllable(Location initLocation) {
		super(initLocation);
	}

	public AbstractSpeedControllable(Location initLocation, double updateTime) {
		super(initLocation, updateTime);
	}

	public final void addControl(ISpeedControl controller) {
		assert controller != null;
		speedControls.add(controller);
	}

	public final void controlSpeed(Speed speed) {
		setSpeed(speed);
	}

	public final void removeControl(ISpeedControl controller) {
		speedControls.remove(controller);
	}

	/**
	 * Implements the normal laws of motion. It needs to be called at every
	 * update
	 * 
	 * @param time
	 * @throws ControlException
	 */
	@Override
	public void updateLocation(double time) throws ControlException {
		for (final ISpeedControl control : speedControls) {
			control.control(time);
		}
		super.updateLocation(time);
	}
}
