package yaes.framework.entity.vehicle;

import yaes.framework.agent.IAgent;
import yaes.framework.entity.control.AbstractSpeedControllable;
import yaes.framework.entity.control.ControlException;
import yaes.framework.entity.control.ISpeedControl;
import yaes.framework.entity.control.speedcontrol.PathFollowControl;
import yaes.framework.world.map.IPathRecorder;
import yaes.framework.world.map.Location;
import yaes.framework.world.map.Path;
import yaes.framework.world.map.PlannedPath;
import yaes.ui.text.TextUi;

public class Vehicle extends AbstractSpeedControllable implements IPathRecorder {
	private String name;
	private IAgent agent;
	private double time;
	private final PlannedPath path = null;
	private ISpeedControl pathControl = null;
	private boolean isRecording = false;
	private Path recordedPath;
	private boolean isStopped;

	public Vehicle(Location initLocation, String name) {
		super(initLocation);
		this.name = name;
		this.isRecording = false;
		this.isStopped = true;
	}

	public IAgent getAgent() {
		return agent;
	}

	public String getName() {
		return name;
	}

	public Path getRecordedPath() {
		return recordedPath;
	}

	public boolean isStopped() {
		return isStopped;
	}

	public void setAgent(IAgent agent) {
		this.agent = agent;
	}

	/**
	 * Specifies the movement of the vehicle.
	 * 
	 * @param path
	 * @param speed
	 */
	public void setMovement(PlannedPath path, double speed) {
		setLocation(path.getSource());
		final ISpeedControl control = new PathFollowControl(path, speed, time);
		control.setControllable(this);
		this.addControl(control);
		isStopped = false;
	}

	public void setName(String name) {
		this.name = name;
	}

	public void startPathRecording() {
		isRecording = true;
		recordedPath = new Path();
	}

	/**
	 * Stops the movement of the vehicle and removes the control
	 * 
	 * @return
	 */
	public void stopMovement() {
		if (isStopped) {
			return;
		}
		removeControl(pathControl);
		pathControl = null;
		isStopped = true;
	}

	public void stopPathRecording() {
		isRecording = false;
		recordedPath = null;
	}

	/**
	 * Updates the location of the agent and calls the associated agent (if
	 * any).
	 * 
	 * @param time
	 * @throws ControlException
	 */
	public void update(double time) throws ControlException {
		this.time = time;
		updateLocation(time);
		if (isRecording) {
			recordedPath.addLocation(getLocation());
		}
		// check if we are at the destination
		if ((pathControl != null)
				&& getLocation().equals(path.getDestination())) {
			TextUi.println("Stop");
			removeControl(pathControl);
			pathControl = null;
			isStopped = true;
		}
		if (agent != null) {
			agent.action();
		}
	}
}
