package yaes.ui.plot;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import yaes.framework.simulation.RandomVariable;
import yaes.framework.simulation.SimulationOutput;
import yaes.util.FileWritingUtil;

/**
 * Describes a classical plot
 */
public class PlotDescription {
	private final String title = "";
	private String xLabel = "";
	private String yLabel = "";
	private final List<PlotLineDescription> plds = new ArrayList<PlotLineDescription>();

	public PlotDescription(String xLabel, String yLabel) {
		this.xLabel = xLabel;
		this.yLabel = yLabel;
	}

	/**
	 * Adds a plot line (a value for google to the plot.
	 * 
	 * @param pld
	 */
	public void addPlotLine(PlotLineDescription pld) {
		plds.add(pld);
	}

	/**
	 * creates a simulation plot from the simulation outputs and the
	 * descriptions
	 * 
	 * @return
	 */
	SimulationPlot createSimulationPlot() {
		// adds this to a simulation plot
		final SimulationPlot plot = new SimulationPlot();
		for (final PlotLineDescription pld : plds) {
			final ArrayList<Double> xValues = new ArrayList<Double>();
			final ArrayList<Double> yValues = new ArrayList<Double>();
			for (final SimulationOutput element : pld.getData()) {
				if (pld.contains(element)) {
					xValues.add(new Double(element.getValue(pld.getXVar(), pld
							.getXVarProbe())));
					yValues.add(new Double(element.getValue(pld.getYVar(), pld
							.getYVarProbe())));
				}
			}
			plot.addOnePlot(pld.getLabel(), yValues, "-");
			plot.setXPoints(xValues);
		}
		return plot;
	}

	/**
	 * Generates the plot into a Matlab compatible file
	 * 
	 * in fact it creates a SimulationPlot object.
	 * 
	 * @param file
	 * @throws IOException
	 */
	public void generate(File file) throws IOException {
		final SimulationPlot plot = createSimulationPlot();
		plot.generateMatlabPlotFileWithTitleLabelLegend(file, title, xLabel,
				yLabel);
	}

	/**
	 * Generates a cummulative distribution plot
	 * 
	 * @param file
	 * @throws IOException
	 */
	public void generateCummulativeDistributionPlot(File file)
			throws IOException {
		final SimulationPlot plot = createSimulationPlot();
		final StringBuffer buffer = plot.getCummulativeDistributionPlot();
		FileWritingUtil.writeToTextFile(file, buffer.toString());
	}

	/**
	 * 
	 * This function is created by Xin on Feb 14, 2005
	 * 
	 * Unfortunately the interface of this function is different from everything
	 * else. It needs to be cleared up and integrated better with the system.
	 * 
	 */
	public void generateErrorBarPlot(File file) throws IOException {
		final SimulationPlot plot = new SimulationPlot();
		for (final PlotLineDescription pld : plds) {
			final ArrayList<Double> xValues = new ArrayList<Double>();
			final ArrayList<Double> yValues = new ArrayList<Double>();
			final ArrayList<Double> yRadiusValues = new ArrayList<Double>();
			for (final SimulationOutput element : pld.getData()) {
				if (pld.contains(element)) {
					xValues.add(new Double(element.getValue(pld.getXVar(), pld
							.getXVarProbe())));
					yValues.add(new Double(element.getValue(pld.getYVar(), pld
							.getYVarProbe())));
					// yRadiusValues.add(new Double(element.getValue(pld
					// .getYVar(), pld.getYRadiusVarType())));
					yRadiusValues
							.add(new Double(element.getValue(pld.getYVar(),
									RandomVariable.Probe.CONFINT_RADIUS)));
				}
			}
			plot
					.addOneErrorBarPlot(pld.getLabel(), yValues, yRadiusValues,
							"-");
			plot.setXPoints(xValues);
		}
		plot.generateMatlabErrorBarPlotFileWithTitleLabelLegend(file, title,
				xLabel, yLabel);
	}
}
