/*
 * Created on Sep 7, 2004
 *
 * To change the template for this generated file go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
package yaes.ui.plot;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import yaes.util.FileWritingUtil;

/**
 * @author Lotzi Boloni
 * 
 * This is the class which actually knows about matlab format etc.
 * 
 */
public class SimulationPlot {
	private double[] xPoints;
	private final ArrayList<String> plotNames = new ArrayList<String>();
	private final ArrayList<double[]> values = new ArrayList<double[]>();
	private final ArrayList<String> plotLineTypes = new ArrayList<String>();
	private final ArrayList<String> yValueStrings = new ArrayList<String>();
	private final String sign[] = { "*", "+", "s", "x", "d", ".", "o", "v",
			"^", "<", ">", "p", "h", };
	private final String type[] = { "-", "--", ":", "-.", "--." };
	private final int count = 0;

	public SimulationPlot() {
	}

	public void addOneErrorBarPlot(String plotName,
			ArrayList<Double> plotValues, ArrayList<Double> plotRadiusValues,
			String plotLineType) {
		plotNames.add(plotName);
		values.add(extractDoubleArray(plotValues));
		values.add(extractDoubleArray(plotRadiusValues));
		plotLineTypes.add(plotLineType);
	}

	public void addOnePlot(String plotName, ArrayList<Double> plotValues,
			String plotLineType) {
		addOnePlot(plotName, extractDoubleArray(plotValues), plotLineType);
	}

	public void addOnePlot(String plotName, double[] plotValues) {
		plotNames.add(plotName);
		values.add(plotValues);
		plotLineTypes.add(sign[count]);
	}

	public void addOnePlot(String plotName, double[] plotValues,
			String plotLineType) {
		plotNames.add(plotName);
		values.add(plotValues);
		plotLineTypes.add(plotLineType);
	}

	/**
	 * This is the one used by Majid, for relative distances
	 * 
	 * @param plotName
	 * @param plotValues
	 * @param plotLineType
	 * @param yValueString
	 */
	public void addOneRelativeDistancePlot(String plotName,
			double[] plotValues, String plotLineType, String yValueString) {
		plotNames.add(plotName);
		values.add(plotValues);
		plotLineTypes.add(plotLineType);
		yValueStrings.add(yValueString);
	}

	private double[] extractDoubleArray(ArrayList<Double> list) {
		final double arrayValues[] = new double[list.size()];
		int i = 0;
		for (final Double element : list) {
			arrayValues[i] = element.doubleValue();
			i++;
		}
		return arrayValues;
	}

	/**
	 * 
	 * FIXME: generate a more readable Matlab file...
	 * 
	 * Creates a Matlab error back plot with a title, a label and a legend.
	 * 
	 * @param file
	 * @param title
	 * @param xLabel
	 * @param yLabel
	 * @throws IOException
	 */
	public void generateMatlabErrorBarPlotFileWithTitleLabelLegend(File file,
			String title, String xLabel, String yLabel) throws IOException {
		final StringBuffer buffer = new StringBuffer();
		buffer.append(getXValuesString());
		buffer.append(getValuesString());
		buffer.append("hold off\n");
		for (int i = 0; i != values.size(); i++) {
			// buffer.append("plot(xvalues, value_" + i + ", '" + color[i / 2]
			// + type[i / 2] + "')\n");
			buffer.append("plot(xvalues, value_" + i);
			buffer.append(",'LineStyle','" + type[i / 2] + "'");
			buffer.append(")\n");
			if (i == 0) {
				buffer.append("hold on\n");
			}
			i = i + 1;
		}
		// generate the legend
		buffer.append("legend(");
		for (int i = 0; i != plotNames.size(); i++) {
			buffer.append("'" + plotNames.get(i) + "'");
			if (i != plotNames.size() - 1) {
				buffer.append(", ");
			}
		}
		// generate the errorbars
		buffer.append(")\n");
		buffer.append("hold on\n");
		for (int i = 0; i != values.size(); i++) {
			// buffer.append("errorbar(xvalues, value_" + i + ", value_" + (i +
			// 1)
			// + ",'" + color[i / 2] + type[i / 2] + "')\n");
			buffer
					.append("errorbar(xvalues, value_" + i + ", value_"
							+ (i + 1));
			buffer.append(",'LineStyle','" + type[i / 2] + "'");
			buffer.append(")\n");
			// buffer.append("hold on\n");
			i = i + 1;
		}
		buffer.append("title('" + title + "')\n");
		buffer.append("hold on\n");
		buffer.append("xlabel('" + xLabel + "')\n");
		buffer.append("hold on\n");
		buffer.append("ylabel('" + yLabel + "')\n");
		FileWritingUtil.writeToTextFile(file, buffer.toString());
	}

	public void generateMatlabPlotFile(File file) throws IOException {
		final StringBuffer buffer = new StringBuffer();
		buffer.append(getXValuesString());
		buffer.append(getValuesString());
		buffer.append("hold off\n");
		for (int i = 0; i != values.size(); i++) {
			buffer.append("plot(xvalues, value_" + i + ", '"
					+ plotLineTypes.get(i) + "')\n");
			buffer.append("hold on\n");
		}
		FileWritingUtil.writeToTextFile(file, buffer.toString());
	}

	/**
	 * Creates a matlab file
	 * 
	 * @param fileName
	 * @throws IOException
	 */
	public void generateMatlabPlotFileWithLegend(File file) throws IOException {
		final StringBuffer buffer = new StringBuffer();
		buffer.append(getXValuesString());
		buffer.append(getValuesString());
		buffer.append("hold off\n");
		for (int i = 0; i != values.size(); i++) {
			buffer.append("plot(xvalues, value_" + i + ", 'b-" + sign[i]
					+ "')\n");
			buffer.append("hold on\n");
		}
		// generate the legend
		buffer.append("legend(");
		for (int i = 0; i != plotNames.size(); i++) {
			buffer.append("'" + plotNames.get(i) + "'");
			if (i != plotNames.size() - 1) {
				buffer.append(", ");
			}
		}
		buffer.append(")\n");
		FileWritingUtil.writeToTextFile(file, buffer.toString());
	}

	// This function is created by Xin on Feb 14, 2005
	public void generateMatlabPlotFileWithTitleLabelLegend(File file,
			String title, String xLabel, String yLabel) throws IOException {
		final StringBuffer buffer = new StringBuffer();
		buffer.append(getXValuesString());
		buffer.append(getValuesString());
		buffer.append("hold off\n");
		for (int i = 0; i != values.size(); i++) {
			buffer.append("plot(xvalues, value_" + i + ", 'b-" + sign[i]
					+ "')\n");
			buffer.append("hold on\n");
		}
		// generate the legend
		buffer.append("legend(");
		for (int i = 0; i != plotNames.size(); i++) {
			buffer.append("'" + plotNames.get(i) + "'");
			if (i != plotNames.size() - 1) {
				buffer.append(", ");
			}
		}
		buffer.append(")\n");
		buffer.append("hold on\n");
		buffer.append("title('" + title + "')\n");
		buffer.append("hold on\n");
		buffer.append("xlabel('" + xLabel + "')\n");
		buffer.append("hold on\n");
		buffer.append("ylabel('" + yLabel + "')\n");
		FileWritingUtil.writeToTextFile(file, buffer.toString());
	}

	/**
	 * Used by Majid, for relative distances.
	 * 
	 * @param fileName
	 * @throws IOException
	 */
	public void generateMatlabRelativeDistancePlotFile(File file)
			throws IOException {
		final StringBuffer buffer = new StringBuffer();
		buffer.append(getXValuesString());
		buffer.append(getValuesString());
		buffer.append("hold off\n");
		for (int i = 0; i != values.size(); i++) {
			buffer.append("plot(xvalues, value_" + i + yValueStrings.get(i)
					+ ", '" + plotLineTypes.get(i) + "')\n");
			buffer.append("hold on\n");
		}
		FileWritingUtil.writeToTextFile(file, buffer.toString());
	}

	/**
	 * Creates a plot for the cummulative distribution.
	 * 
	 * @return
	 */
	public StringBuffer getCummulativeDistributionPlot() {
		final StringBuffer buffer = new StringBuffer();
		buffer.append(getXValuesString());
		buffer.append(getValuesString());
		buffer.append("hold on\n");
		for (int i = 0; i != values.size(); i++) {
			buffer.append("[f_" + i + " xf] = ecdf(value_" + i + ")\n");
			buffer.append("stairs(xf, f_" + i + ",'" + type[i] + "')\n");
		}
		buffer.append("legend(");
		for (int i = 0; i != plotNames.size() - 1; i++) {
			buffer.append("'" + plotNames.get(i) + "', ");
		}
		buffer.append("'" + plotNames.get(plotNames.size() - 1) + "')\n");
		buffer.append("hold off\n");
		return buffer;
	}

	/**
	 * Gets the string for the Y values.
	 * 
	 * @return
	 */
	private String getValuesString() {
		final StringBuffer buffer = new StringBuffer();
		int count = 0;
		for (double[] name : values) {
			final double[] element = name;
			buffer
					.append(MatlabUtil.getMatlabVector("value_" + count,
							element));
			count++;
		}
		return buffer.toString();
	}

	/**
	 * Gets the string for the X values.
	 * 
	 * @return
	 */
	private String getXValuesString() {
		return MatlabUtil.getMatlabVector("xvalues", xPoints);
	}

	public void setXPoints(ArrayList<Double> list) {
		this.xPoints = extractDoubleArray(list);
	}

	public void setXPoints(double[] xPoints) {
		this.xPoints = xPoints;
	}
}
