package yaes.ui.propertysheet;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingConstants;

import yaes.ui.propertysheet.editors.CustomEditorWidget;

public class EditListWidget extends JPanel implements CustomEditorWidget,
		ActionListener {
	class ListEditActionListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			final JButton button = (JButton) e.getSource();
			if (button.getText().equals(
					EditListWidget.buttonLabels[ADD_TO_LIST])) {
				doAdd();
			} else if (button.getText().equals(
					EditListWidget.buttonLabels[REMOVE_FROM_LIST])) {
				doRemove();
			} else if (button.getText().equals(
					EditListWidget.buttonLabels[REMOVE_ALL])) {
				doRemoveAll();
			} else if (button.getText().equals(
					EditListWidget.buttonLabels[MOVE_UP])) {
				moveUp();
			} else if (button.getText().equals(
					EditListWidget.buttonLabels[MOVE_DOWN])) {
				moveDown();
			}
		}
	}

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private final static String[] buttonLabels = { "Add", "Remove",
			"Remove All", "Move Up", "Move Down" };
	private final static String[] buttonToolTips = {
			"Add new item to the list", "Remove selected item from the list",
			"Remove all items from the list", "Move selected item up",
			"Move selected item down" };
	private final static char[] buttonMnemonics = { 'A', 'R', 'm', 'U', 'D' };
	private final static ImageIcon iconAdd = new ImageIcon(
			"com/cplane/ic/gui/resources/smallimages/Add16.gif");
	private final static ImageIcon iconRemove = new ImageIcon(
			"com/cplane/ic/gui/resources/smallimages/remove.png");
	private final static ImageIcon iconRemoveAll = new ImageIcon(
			"com/cplane/ic/gui/resources/smallimages/trash.png");
	private final static ImageIcon iconUp = new ImageIcon(
			"com/cplane/ic/gui/resources/smallimages/up.png");
	private final static ImageIcon iconDown = new ImageIcon(
			"com/cplane/ic/gui/resources/smallimages/down.png");
	private final static ImageIcon[] buttonIcons = { EditListWidget.iconAdd,
			EditListWidget.iconRemove, EditListWidget.iconRemoveAll,
			EditListWidget.iconUp, EditListWidget.iconDown };
	private TableList tableList;
	private final int ADD_TO_LIST = 0;
	private final int REMOVE_FROM_LIST = 1;
	private final int REMOVE_ALL = 2;
	private final int MOVE_UP = 3;
	private final int MOVE_DOWN = 4;
	private String title = "Edit List of Values";

	public EditListWidget(Vector list) {
		init(list);
	}

	public void actionOk() {
	}

	public void actionPerformed(ActionEvent e) {
		final String stringSource = ((Component) e.getSource()).getName();
		if (stringSource.equals(DialogFrame.NAME_OK)) {
			tableList.trim();
		}
	}

	private void createButtons(JPanel lowerButtonPanel, JPanel upperButtonPanel) {
		for (int i = 0; i < EditListWidget.buttonLabels.length; i++) {
			final JButton button = new JButton(EditListWidget.buttonLabels[i]);
			button.setToolTipText(EditListWidget.buttonToolTips[i]);
			button.setMnemonic(EditListWidget.buttonMnemonics[i]);
			button.setIcon(EditListWidget.buttonIcons[i]);
			button.setHorizontalAlignment(SwingConstants.LEFT);
			button.addActionListener(new ListEditActionListener());
			if ((i == MOVE_UP) || (i == MOVE_DOWN)) {
				lowerButtonPanel.add(button);
			} else {
				upperButtonPanel.add(button);
			}
		}
	}

	private void doAdd() {
		tableList.addRow();
	}

	private void doRemove() {
		final int index = tableList.getSelectedRow();
		tableList.removeRow(index);
	}

	private void doRemoveAll() {
		tableList.removeAll();
	}

	public Vector getList() {
		return tableList.getModifiedList();
	}

	public String getWidgetTitle() {
		return title;
	}

	/*
	 * public EditListWidget() { init(); }
	 */
	private void init(Vector dataList) {
		final JPanel panel = new JPanel();
		tableList = new TableList(dataList);
		final JScrollPane scrollPane = new JScrollPane(tableList);
		final BorderLayout borderLayout = new BorderLayout();
		panel.setLayout(borderLayout);
		panel.setBorder(BorderFactory.createCompoundBorder(BorderFactory
				.createTitledBorder(BorderFactory.createEtchedBorder(),
						"Edit List of Values"), BorderFactory
				.createEmptyBorder(10, 10, 10, 10)));
		panel.add(scrollPane, BorderLayout.CENTER);
		final JPanel buttonPanel = new JPanel();
		final BorderLayout bl = new BorderLayout();
		buttonPanel.setLayout(bl);
		final JPanel upperButtonPanel = new JPanel();
		final JPanel lowerButtonPanel = new JPanel();
		final GridLayout gl1 = new GridLayout(3, 1);
		gl1.setVgap(5);
		final GridLayout gl2 = new GridLayout(2, 1);
		gl2.setVgap(5);
		upperButtonPanel.setLayout(gl1);
		lowerButtonPanel.setLayout(gl2);
		createButtons(lowerButtonPanel, upperButtonPanel);
		buttonPanel.add(upperButtonPanel, BorderLayout.NORTH);
		buttonPanel.setBorder(BorderFactory.createEmptyBorder(0, 10, 0, 0));
		buttonPanel.add(lowerButtonPanel, BorderLayout.SOUTH);
		panel.add(buttonPanel, BorderLayout.EAST);
		final BorderLayout bl2 = new BorderLayout();
		this.setLayout(bl2);
		this.add(panel);
		this.setPreferredSize(new Dimension(400, 300));
	}

	private void moveDown() {
		final int index = tableList.getSelectedRow();
		tableList.moveDown(index);
	}

	private void moveUp() {
		final int index = tableList.getSelectedRow();
		tableList.moveUp(index);
	}

	public void setErrorPresenter(ErrorPresenter errorPresenter) {
		tableList.setErrorPresenter(errorPresenter);
	}

	public void setList(Vector list) {
		tableList.setList(list);
	};

	public void setWidgetTitle(String title) {
		this.title = title;
	}
}