package yaes.ui.propertysheet;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Map;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JRootPane;

/*Extended Property Sheet
 *wraps dynamic addition and removal of property
 */
public class ExPropertySheet extends PropertySheet {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	protected DynamicPropertyManager propManager;
	protected ExPropertyTableModel model = null;
	protected JButton btnAddProp = new JButton();
	protected JButton btnRemoveProp = new JButton();
	private final ImageIcon AddIcon = new ImageIcon("Add16.gif");
	private final ImageIcon DeleteIcon = new ImageIcon("Delete16.gif");

	public ExPropertySheet() {
		super();
	}

	public ExPropertySheet(Map map, DynamicPropertyManager propManager) {
		createExPropertySheet(map, propManager);
	}

	public void addNewProperty(String name, Object value, int access) {
		model.addRow(name, value);
		// add code to set an access
	}

	protected void createExPropertySheet(Map map,
			DynamicPropertyManager propManager) {
		this.propManager = propManager;
		model = new ExPropertyTableModel(map);
		createPropertySheet(model);
	}

	protected void createExPropertySheet(Map map,
			DynamicPropertyManager propManager,
			AttributeUtilityManager attrManager) {
		this.propManager = propManager;
		model = new ExPropertyTableModel(map, attrManager);
		createPropertySheet(model);
	}

	public Map getAllProperty() {
		return model.properties;
	}

	@Override
	protected void initButtons() {
		super.initButtons();
		btnAddProp.setIcon(AddIcon);
		btnRemoveProp.setIcon(DeleteIcon);
		btnAddProp.setToolTipText("Add Property");
		btnRemoveProp.setToolTipText("Remove Property");
		btnAddProp.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				final JRootPane pane = ((JComponent) e.getSource())
						.getRootPane();
				propManager.processAddPropertyRequest(pane.getParent());
				final String propertyName = propManager.getPropertyName();
				if ((propertyName == null) || propertyName.equals("")) {
					return;
				}
				addNewProperty(propManager.getPropertyName(), propManager
						.getPropertyValue(), propManager.getPropertyAccess());
			}
		});
		btnRemoveProp.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				final JRootPane pane = ((JComponent) e.getSource())
						.getRootPane();
				final int row = tableSheet.getSelectedRow();
				if (row >= 0) {
					final String propName = model.propertyForRow(row);
					if (propManager.processRemovePropertyRequest(pane
							.getParent(), propName)) {
						removeProperty();
					}
				}
			}
		});
	}

	@Override
	protected void initToolBar() {
		toolBar.add(btnSortByName);
		toolBar.addSeparator();
		toolBar.add(btnSortByType);
		toolBar.addSeparator();
		toolBar.add(btnAddProp);
		toolBar.addSeparator();
		toolBar.add(btnRemoveProp);
	}

	public void removeAllProperty() {
		final int count = model.getPropertyRowCount();
		if (count <= 0) {
			return;
		}
		for (int i = 0; i < count; i++) {
			model.removeRow(0); // always remove the first one; error:
			// model.removeRow(i);
		}
	}

	public void removeProperty() {
		model.removeRow(tableSheet.getSelectedRow());
	}

	public void setAddPropToolTipText(String str) {
		btnAddProp.setToolTipText(str);
	}

	protected void setExModel(ExPropertyTableModel pModel) {
		model = pModel;
	}

	public void setRemovePropToolTipText(String str) {
		btnRemoveProp.setToolTipText(str);
	}
}
