package yaes.ui.propertysheet;

import java.awt.Component;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.text.ParseException;
import java.util.EventObject;

import javax.swing.AbstractCellEditor;
import javax.swing.JFormattedTextField;
import javax.swing.JTable;
import javax.swing.table.TableCellEditor;

public class FormattedCellEditor extends AbstractCellEditor implements
		TableCellEditor {
	/**
	 * 
	 */
	private static final long serialVersionUID = -1289248975512484878L;
	/**
	 * 
	 */
	private JFormattedTextField field;
	private final String className;

	public FormattedCellEditor(String className) {
		this.className = className;
		createFormattedField();
		field.addKeyListener(new KeyAdapter() {
			@Override
			public void keyPressed(KeyEvent e) {
				if (e.getKeyCode() == KeyEvent.VK_ENTER) {
					completeEdit();
					final String str = field.getValue().toString();
					field.setCaretPosition(str.length());
					FormattedCellEditor.this.fireEditingStopped();
				}
			}
		});
		field.addFocusListener(new FocusAdapter() {
			@Override
			public void focusLost(FocusEvent e) {
				completeEdit();
				// FormattedCellEditor.this.fireEditingStopped();
			}
		});
	}

	public void completeEdit() {
		final int fb = field.getFocusLostBehavior();
		if ((fb == JFormattedTextField.COMMIT)
				|| (fb == JFormattedTextField.COMMIT_OR_REVERT)) {
			try {
				field.commitEdit();
				// Give it a chance to reformat.
				field.setValue(field.getValue());
			} catch (final ParseException pe) {
				if (fb == JFormattedTextField.COMMIT_OR_REVERT) {
					field.setValue(field.getValue());
				}
			}
		} else if (fb == JFormattedTextField.REVERT) {
			field.setValue(field.getValue());
		}
	}

	private void createFormattedField() {
		if (className.equals("java.lang.Integer")) {
			final JFormattedTextField.AbstractFormatter intFormatter = new JFormattedTextField.AbstractFormatter() {
				/**
				 * 
				 */
				private static final long serialVersionUID = 1L;

				@Override
				public Object stringToValue(String text) throws ParseException {
					Integer value = null;
					try {
						value = Integer.valueOf(text);
					} catch (NumberFormatException exc) {
						throw new ParseException("Unparseable number.", 0);
					}
					return value;
				}

				@Override
				public String valueToString(Object value) throws ParseException {
					if (value == null) {
						throw new ParseException("Parse Error.", 0);
					}
					return value.toString();
				}
			};
			field = new JFormattedTextField(intFormatter);
		} else if (className.equals("java.lang.String")) {
			final JFormattedTextField.AbstractFormatter stringFormatter = new JFormattedTextField.AbstractFormatter() {
				/**
				 * 
				 */
				private static final long serialVersionUID = 1L;

				@Override
				public Object stringToValue(String text) throws ParseException {
					String value = "";
					if (text.length() > 0) {
						value = text.trim();
					}
					if (value.length() == 0) {
						throw new ParseException("Unparseable entry.", 0);
					}
					return value;
				}

				@Override
				public String valueToString(Object value) throws ParseException {
					if (value == null) {
						throw new ParseException("Parse Error.", 0);
					}
					return value.toString();
				}
			};
			field = new JFormattedTextField(stringFormatter);
		} else {
			field = new JFormattedTextField();
		}
	}

	public Object getCellEditorValue() {
		completeEdit();
		return field.getValue();
	}

	public Component getTableCellEditorComponent(JTable table, Object value,
			boolean isSelected, int row, int column) {
		field.setValue(table.getModel().getValueAt(row, column));
		return field;
	}

	@Override
	public boolean isCellEditable(EventObject event) {
		if (event instanceof MouseEvent) {
			final MouseEvent mouseEvent = (MouseEvent) event;
			if (mouseEvent.getClickCount() > 1) {
				return true;
			}
		}
		return false;
	}
}
