package yaes.ui.propertysheet;

import java.beans.PropertyChangeListener;
import java.lang.reflect.InvocationTargetException;
import java.util.Vector;

/**
 * Model defining the right behaviour of property. This model should be used for
 * communication with PropertyBean bean.
 * 
 */
public interface PropertyModel {
	/** Name of the 'value' property. */
	public static final String PROP_VALUE = "value"; // NOI18N

	/**
	 * Add listener to change of the value.
	 */
	public void addPropertyChangeListener(PropertyChangeListener l);

	public Vector<PropertyChangeListener> getChangeListeners();

	/**
	 * The class of the property editor or <CODE>null</CODE> if default
	 * property editor should be used.
	 */
	@SuppressWarnings("unchecked")
	public Class getPropertyEditorClass();

	/**
	 * The class of the property.
	 */
	@SuppressWarnings("unchecked")
	public Class getPropertyType();

	public String[] getTags();

	/**
	 * Getter for current value of a property.
	 */
	public Object getValue() throws InvocationTargetException;

	public boolean isPropertyName();

	public boolean isValueReadOnly();

	/**
	 * Remove listener to change of the value.
	 */
	public void removePropertyChangeListener(PropertyChangeListener l);

	public void setChangeListeners(Vector<PropertyChangeListener> v);

	/**
	 * Setter for a value of a property.
	 * 
	 * @param v
	 *            the value
	 * @exeception InvocationTargetException
	 */
	public void setValue(Object v) throws InvocationTargetException;
}
