package yaes.ui.propertysheet;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.beans.PropertyEditor;
import java.util.Vector;

/**
 * This lightweight component encapsulates calling of propertyEditor.paintValue
 * (Graphics g, Rectangle r) method in special Component.
 * 
 * @author Jan Jancura
 * @version 0.16
 */
class PropertyShow extends javax.swing.JPanel {
	/** generated Serialized Version UID */
	static final long serialVersionUID = -8663903931982719530L;
	/** Link to the property editor */
	private PropertyEditor propertyEditor;
	/** Standart helper variable. */
	private final Vector listeners = new Vector(1, 5);

	/**
	 * Constructs new PropertyShow for specified PropertyEditor.
	 * 
	 * @param PropertyEditor
	 *            aPropertyEditor proper property editor
	 */
	public PropertyShow(PropertyEditor aPropertyEditor) {
		propertyEditor = aPropertyEditor;
		setDoubleBuffered(false);
		setOpaque(true);
	}

	/** Adds <code>SheetButtonListener</code>. */
	public void addSheetButtonListener(SheetButtonListener sheetListener) {
		listeners.addElement(sheetListener);
	}

	/** Fires sheet button was cliced. */
	void fireSheetButtonClicked(ActionEvent e) {
		final Vector l = (Vector) listeners.clone();
		int i;
		final int k = l.size();
		for (i = 0; i < k; i++) {
			((SheetButtonListener) l.elementAt(i)).sheetButtonClicked(e);
		}
	}

	/**
	 * Standart method for painting component.
	 */
	@Override
	public void paintComponent(Graphics g) {
		final Dimension sz = getSize();
		final Color color = g.getColor();
		g.setColor(getBackground());
		g.fillRect(0, 0, sz.width, sz.height);
		final Rectangle r = new Rectangle();
		r.setBounds(4, 0, sz.width - 4, sz.height);
		// XXX Use foreground color of PropertyShow to reflect r/w v. r/o
		// property, see PropertyPanel -> use of canWrite property.
		g.setColor(getForeground());
		propertyEditor.paintValue(g, r);
		g.setColor(color);
		if (hasFocus()) {
			g.setColor(javax.swing.UIManager.getColor("Button.focus")); // NOI18N
			g.drawRect(1, 1, sz.width - 3, sz.height - 3);
		}
	}

	/** Removes <code>SheetButtonListener</code>. */
	public void removeSheetButtonListener(SheetButtonListener sheetListener) {
		listeners.removeElement(sheetListener);
	}

	/** Sets <code>propertyEditor</code>. */
	public void setEditor(PropertyEditor aEditor) {
		propertyEditor = aEditor;
	}

	/**
	 * Sets value of showen property.
	 */
	public void setValue(Object newValue) {
		propertyEditor.setValue(newValue);
	}
}
