package yaes.ui.simulationcontrol;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import yaes.ui.text.TextUi;

public class SimulationControlPanel extends JPanel implements ActionListener,
		ChangeListener {
	public enum State {
		INITIAL, READY_TO_RUN, RUNNING, FINISHED, POSTPROCESSED
	};

	private static final long serialVersionUID = 5148822679306814062L;
	private final SimulationControlGui theGui;
	private final JPanel pnlButtons;
	private final JPanel pnlSlider;
	// buttons
	private final JButton btnSetup;
	private final JButton btnStep;
	private final JButton btnStop;
	private final JButton btnRun;
	private final JButton btnPostProcess;
	// 
	private final JTextField tfStatus;
	private final JSlider sldDelay;
	private State state = State.INITIAL;
	private final PnlSimulationOutput pnlOutput;

	public SimulationControlPanel(SimulationControlGui theGui) {
		this.theGui = theGui;
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		// button panel
		pnlButtons = new JPanel();
		pnlButtons.setLayout(new BoxLayout(pnlButtons, BoxLayout.X_AXIS));
		add(pnlButtons);
		btnSetup = addButton("Setup");
		btnStep = addButton("Step");
		btnRun = addButton("Run");
		btnStop = addButton("Stop");
		btnPostProcess = addButton("Postprocess");
		tfStatus = UiHelper.createDisplayField(20);
		pnlButtons.add(tfStatus);
		// slider panel
		pnlSlider = new JPanel();
		pnlSlider.setLayout(new BoxLayout(pnlSlider, BoxLayout.X_AXIS));
		add(pnlSlider);
		sldDelay = new JSlider(SwingConstants.HORIZONTAL, 0, 2000, theGui
				.getRunningDelay());
		sldDelay.addChangeListener(this);
		// Turn on labels at major tick marks.
		sldDelay.setMajorTickSpacing(500);
		sldDelay.setMinorTickSpacing(50);
		sldDelay.setPaintTicks(true);
		sldDelay.setPaintLabels(true);
		pnlSlider.add(sldDelay);

		// wrap up
		state = State.INITIAL;
		setButtonBasedOnState();
		pnlOutput = new PnlSimulationOutput();
		add(pnlOutput);
	}

	public void actionPerformed(ActionEvent event) {
		if (event.getSource() == btnSetup) {
			actionSetup();
			state = State.READY_TO_RUN;
			setButtonBasedOnState();
			return;
		}
		if (event.getSource() == btnStep) {
			actionStep();
			state = State.READY_TO_RUN;
			setButtonBasedOnState();
			return;
		}
		if (event.getSource() == btnRun) {
			actionRun();
			state = State.RUNNING;
			setButtonBasedOnState();
			return;
		}
		if (event.getSource() == btnStop) {
			actionStop();
			state = State.READY_TO_RUN;
			setButtonBasedOnState();
			return;
		}
		if (event.getSource() == btnPostProcess) {
			actionPostProcess();
			state = State.POSTPROCESSED;
			setButtonBasedOnState();
			return;
		}
	}

	public void actionPostProcess() {
		TextUi.println("actionPostProcess called");
		theGui.setNextCommand(SimulationControlGui.Command.PostProcess);
	}

	public void actionRun() {
		TextUi.println("actionRun called");
		theGui.setNextCommand(SimulationControlGui.Command.Run);
	}

	public void actionSetup() {
		TextUi.println("actionSetup called");
		theGui.setNextCommand(SimulationControlGui.Command.Setup);
	}

	public void actionStep() {
		TextUi.println("actionStep called");
		theGui.setNextCommand(SimulationControlGui.Command.Step);
	}

	public void actionStop() {
		TextUi.println("actionStop called");
		theGui.setNextCommand(SimulationControlGui.Command.Stop);
	}

	/**
	 * Adds a button to the button panel
	 * 
	 * @param text
	 * @return
	 */
	private JButton addButton(String text) {
		final JButton button = new JButton(text);
		pnlButtons.add(button);
		pnlButtons
				.add(Box.createHorizontalStrut(UiHelper.INTERBUTTON_DISTANCE));
		button.addActionListener(this);
		button.setEnabled(false);
		return button;
	}

	/**
	 * Turns the buttons on and off.
	 * 
	 */
	private void setButtonBasedOnState() {
		tfStatus.setText(theGui.getStatusString());
		if (theGui.getSimulationOutput() != null) {
			pnlOutput.setSimulationOutput(theGui.getSimulationOutput());
		}
		switch (state) {
		case INITIAL:
			btnSetup.setEnabled(true);
			btnStep.setEnabled(false);
			btnRun.setEnabled(false);
			btnStop.setEnabled(false);
			btnPostProcess.setEnabled(false);
			break;
		case READY_TO_RUN:
			btnSetup.setEnabled(false);
			btnStep.setEnabled(true);
			btnRun.setEnabled(true);
			btnStop.setEnabled(false);
			btnPostProcess.setEnabled(false);
			break;
		case RUNNING:
			btnSetup.setEnabled(false);
			btnStep.setEnabled(false);
			btnRun.setEnabled(false);
			btnStop.setEnabled(true);
			btnPostProcess.setEnabled(false);
			break;
		case FINISHED:
			btnSetup.setEnabled(false);
			btnStep.setEnabled(false);
			btnRun.setEnabled(false);
			btnStop.setEnabled(false);
			btnPostProcess.setEnabled(true);
			break;
		case POSTPROCESSED:
			btnSetup.setEnabled(false);
			btnStep.setEnabled(false);
			btnRun.setEnabled(false);
			btnStop.setEnabled(false);
			btnPostProcess.setEnabled(false);
			break;
		}
	}

	public void stateChanged(ChangeEvent arg0) {
		if (arg0.getSource() == sldDelay) {
			if (!sldDelay.getValueIsAdjusting()) {
				theGui.setRunningDelay(sldDelay.getValue());
			}
		}
	}

	public void update() {
		tfStatus.setText(theGui.getStatusString());
	}
}
