package yaes.ui.visualization;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

/**
 * This will normally appear on the right side of the visual map and allows the
 * inspection of the individual elements.
 * 
 * For the time being: a list on the top, a text area in the bottom.
 * 
 * @author Lotzi Boloni
 * 
 */
public class VisualMapInspector extends JPanel implements ListSelectionListener {

	private static final long serialVersionUID = 3555680915038609439L;
	private final JList list;
	private final DefaultListModel model;
	private Object currentlySelected;
	private JTextArea textArea;
	private List<Object> objects;
	private Map<Object, IInspector> inspectors;
	private Observer observer = new Observer() {

		public void update(Observable arg0, Object arg1) {
			if (arg0 == currentlySelected) {
				updateLongText();
			}
		}
	};

	/**
	 * Create the panel
	 * 
	 * @param parent
	 */
	public VisualMapInspector(VisualMap parent) {
		// initialize the data structures
		objects = new ArrayList<Object>();
		inspectors = new HashMap<Object, IInspector>();
		// visual setup
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		setBorder(BorderFactory.createTitledBorder("Object inspector"));
		list = new JList();
		list.addListSelectionListener(this);
		model = new DefaultListModel();
		list.setModel(model);
		final JScrollPane scrlList = new JScrollPane(list);
		add(scrlList);
		textArea = new JTextArea(20, 20);
		textArea.setEditable(false);
		final JScrollPane scrTextArea = new JScrollPane(textArea);
		add(scrTextArea);
	}

	/**
	 * Adds an object to the inspector -if it is an observable object,
	 * 
	 * @param o
	 */
	public void addObject(Object o, IInspector inspector) {
		model.addElement(inspector.getLineDescriptor(o));
		objects.add(o);
		inspectors.put(o, inspector);
		if (o instanceof Observable) {
			((Observable) o).addObserver(observer);
		}
	}

	/**
	 * FIXME: this is not quite the right way here, because the inspector line
	 * desciptor might have changed
	 * 
	 * @param o
	 */
	public void removeObject(Object o) {
		if (o instanceof Observable) {
			((Observable) o).deleteObserver(observer);
		}
		IInspector inspector = inspectors.get(o);
		if (inspector == null) {
			return;
		}
		model.removeElement(inspector.getLineDescriptor(o));
		objects.remove(o);
	}

	private void updateLongText() {
		final IInspector inspector = inspectors.get(currentlySelected);
		textArea.setText(inspector.getLongDescriptor(currentlySelected));
	}

	/**
	 * The list selection changed. Change the value in the area.
	 * 
	 */
	public void valueChanged(ListSelectionEvent e) {
		if (e.getValueIsAdjusting()) {
			textArea.setText("");
			return;
		}
		final int first = list.getMinSelectionIndex();
		final int second = list.getMaxSelectionIndex();
		if (first != second) {
			textArea.setText("Multiple objects selected");
			return;
		}
		currentlySelected = objects.get(first);
		updateLongText();
	}
}
